<?php
/**
 * Provides functions to get/add/del/alter records in the database
 *
 * $Id: functions.records.php 203 2007-07-20 09:23:51Z rbott $
 *
 * @package ZoneAdmin
 * @author Rudolph Bott
 * @version 0.1
 * @copyright Rudolph Bott
 * @license http://www.gnu.org/licenses/gpl.html GNU General Public License
 */


// vim:tabstop=2:expandtab:shiftwidth=2:filetype=php:syntax:ruler:

/**
 * validate record information
 *
 * this function takes all arguments that functions like
 * add_domain_record() or alter_domain_record() take
 * and checks weather they apear to make sense
 * or not (e.g. no IP adress in a SOA-like record)
 * check-patterns are defined dynamically in config.php
 * to allow easy customisation
 * @param string $name FQDN
 * @param string $type record-type (e.g. A)
 * @param string $content the content (e.g. IP Address)
 * @param integer $prio the MX priority (if unused, set to 0)
 * @param integer $ttl the TTL of this record (optional)
 * @return boolean true or false
 * @version 1.0
 */

function valid_record($name,$type,$content,$prio,$ttl = 0) {
  global $validate_patterns;
  global $cfg;

  if($ttl == 0) {
    $ttl = $cfg["defaultttl"];
  }

  $validated = false;
  if(isset($validate_patterns[$type])) {
    $validated = true;

    if(!preg_match($validate_patterns[$type]["name_regex"],$name)) {
      add_error(ERR_INVAL_NAME,__FUNCTION__);
      $validated = false;
    }

    if(!preg_match($validate_patterns[$type]["content_regex"],$content)) {
      add_error(ERR_INVAL_CONTENT,__FUNCTION__);
      $validated = false;
    }

    if(!preg_match($validate_patterns[$type]["prio_regex"],$prio)) {
      add_error(ERR_INVAL_PRIO,__FUNCTION__);
      $validated = false;
    }

    if(!preg_match($validate_patterns[$type]["ttl_regex"],$ttl)) {
      add_error(ERR_INVAL_TTL,__FUNCTION__);
      $validated = false;
    }
  }

  return $validated;
}


/**
 * strip $ORIGIN from FQDN
 *
 * this function takes a FQDN and a domain as arguments
 * and strips the latter off the FQDN
 * @param string $record FQDN
 * @param string $domain domain-origin
 * @return string record
 * @author Rudolph Bott
 * @copyright Copyright 2007, Rudolph Bott
 * @version V1.0
*/
function strip_origin($record,$domain) {
  $host = substr($record,0,strpos($record,$domain));
  if(substr($host,-1) == ".") {
    $host = substr($host,0,-1);
  }
  return $host;
}

/**
 * translates a record-id to its fqdn *NOT IMPLEMENTED YET*
 *
 * this function looks up a given record-id and returns
 * the matching fully qualified domain name as a string
 * @param integer $id record-id
 * @return string FQDN 
 * @author Rudolph Bott
 * @copyright Copyright 2007, Rudolph Bott
 * @version V1.0
*/
function get_record_from_id($id) {
  global $mysqlcfg;
  $record = "";

  $saveorigin = $cfg["striporigin"];
  $cfg["striporigin"] = false;

  if(is_numeric($id)) {
    $query = "SELECT name FROM ".$mysqlcfg["rectbl"]." WHERE id='".mysql_real_escape_string($id)."'";
    $result = mysql_query($query);
    $row = mysql_fetch_assoc($result);
    $record = $row["name"];
  }

  $cfg["striporigin"] = $saveorigin;

  return $record;
}

/**
 * translates a FQDN to its record-id
 *
 * this function looks up a given FQDN in the
 * database and returns its record-id (0 if not found)
 * optionally, you can pass the record-type as
 * a second parameter
 * @param string $name the FQDN
 * @param string $type optional, record type (e.g. A, NS)
 * @return integer record-id of the given FQDN (or 0) 
 * @author Rudolph Bott
 * @copyright Copyright 2007, Rudolph Bott
 * @version V1.0
*/
function get_id_from_record($name,$type = "ANY",$content = "*") {
  global $mysqlcfg;
  global $cfg;
  $record = array();

  if(!empty($name)) {
    $query = "SELECT id FROM ".$mysqlcfg["rectbl"]." WHERE name='".mysql_real_escape_string($name)."'";
    if($type != "ANY") {
      $query .= " AND type='".$type."'";
    }
    if($content != "*") {
      $query .= " AND content='".$content."'";
    }
    $result = mysql_query($query);
    if(mysql_num_rows($result) != 1) {
      return 0;
    }
    else {
      $id = mysql_fetch_assoc($result);
      mysql_free_result($result);
      return $id["id"];
    }
  }
  return 0;
}

/**
 * retrieve a complete record
 *
 * this function fetches a complete record (FQDN, domain-id,
 * type, content, prio, comment, comment-user, comment-time)
 * from the database
 *
 * optionally the second parameter can be set to true to
 * tell the function to use the removed-records table instead
 * of the active-records table
 * @param integer $id the record-id
 * @param boolean $disabled if true, retrieve record from disabled-records table
 * @return array record structure
 * @author Rudolph Bott
 * @copyright Copyright 2007, Rudolph Bott
 * @version V1.0
*/
function get_domain_record($id,$disabled = false) {
  global $mysqlcfg;
  global $cfg;
  $record = array();

  if($disabled) {
    $domtbl = $mysqlcfg["remdomtbl"];
    $rectbl = $mysqlcfg["remrectbl"];
    $cmttbl = $mysqlcfg["remcmttbl"];
  }
  else {
    $domtbl = $mysqlcfg["domtbl"];
    $rectbl = $mysqlcfg["rectbl"];
    $cmttbl = $mysqlcfg["cmttbl"];
  }

  if(!empty($id) && is_numeric($id)) {
    $query = "SELECT d.name AS domain,r.id,r.domain_id,r.name,r.type,r.prio,r.ttl,r.content,c.msg AS comment,c.when AS comment_when, c.user AS comment_user FROM ".$rectbl." r LEFT JOIN ".$cmttbl." c ON (c.record_id = r.id) LEFT JOIN ".$domtbl." d ON (d.id = r.domain_id) WHERE r.id='".mysql_real_escape_string($id)."'";
    $result = mysql_query($query);
    $record = mysql_fetch_assoc($result);
    mysql_free_result($result);

    if($cfg["striporigin"]) {
      $record["name"] = strip_origin($record["name"],$record["domain"]);
    }
  }

  return $record;
}

/**
 * retrieves all records for a given domain-id
 *
 * this function fetches all records associated
 * with a given domain-id (see get_domain_record() )
 *
 * uses FIND_IN_SET() to create a custom SQL sorting
 * order (SOA,NS;MX,TXT,PTR,A,CNAME)
 *
 * optionally the second parameter can be set to true to
 * tell the function to use the removed-records table instead
 * of the active-records table
 * @param integer $id domain-id
 * @param boolean $disabled if true, retrieve records from disabled-records table
 * @return array 2-dimensional array of record structures 
 * @author Rudolph Bott
 * @copyright Copyright 2007, Rudolph Bott
 * @version V1.0
*/
function get_domain_records($id,$disabled = false) {
  global $mysqlcfg;
  global $cfg;

  $records = array();

  if($disabled) {
    $domtbl = $mysqlcfg["remdomtbl"];
    $rectbl = $mysqlcfg["remrectbl"];
    $cmttbl = $mysqlcfg["remcmttbl"];
  }
  else {
    $domtbl = $mysqlcfg["domtbl"];
    $rectbl = $mysqlcfg["rectbl"];
    $cmttbl = $mysqlcfg["cmttbl"];
  }

  if(!empty($id) && is_numeric($id)) {
    $query = "SELECT d.name AS domain,r.id,r.domain_id,r.name,r.type,r.prio,r.ttl,r.content,FIND_IN_SET(r.type,'SOA,NS,MX,TXT,PTR,A,AAAA,CNAME') AS sortOrder,c.msg AS comment,c.when AS comment_when, c.user AS comment_user FROM ".$rectbl." r LEFT JOIN ".$cmttbl." c ON (c.record_id = r.id) LEFT JOIN ".$domtbl." d ON (d.id = r.domain_id) WHERE r.domain_id='".mysql_real_escape_string($id)."' ORDER BY sortOrder,r.name";
    $result = mysql_query($query);

    while($row = mysql_fetch_assoc($result)) {
      if($cfg["striporigin"]) {
        $row["name"] = strip_origin($row["name"],$row["domain"]);
      }
      $records[] = $row;
    }
    mysql_free_result($result);
  }
  return $records;
}

/**
 * retrieve all records for a given domain-id and record-type
 *
 * 
 * works like get_domain_records(), but only
 * retrieves records with type=$type
 * @param integer $id the domain-id
 * @param string $type the record-type
 * @return array 2-dimensional array of record structures 
 * @author Rudolph Bott
 * @copyright Copyright 2007, Rudolph Bott
 * @version V1.0
*/
function get_domain_records_by_type($id,$type) {
  global $mysqlcfg;
  global $cfg;

  $records = array();

  if(!empty($id) && is_numeric($id)) {
    $query = "SELECT d.name AS domain,r.id,r.domain_id,r.name,r.type,r.prio,r.ttl,r.content,c.msg AS comment,c.when AS comment_when, c.user AS comment_user FROM ".$mysqlcfg["rectbl"]." r LEFT JOIN ".$mysqlcfg["cmttbl"]." c ON (c.record_id = r.id) LEFT JOIN ".$mysqlcfg["domtbl"]." d ON (d.id = r.domain_id) WHERE r.domain_id='".mysql_real_escape_string($id)."' AND r.type='".mysql_real_escape_string($type)."' ORDER BY r.name";
    $result = mysql_query($query);
    echo mysql_error();
    while($row = mysql_fetch_assoc($result)) {
      if($cfg["striporigin"]) {
        $row["name"] = strip_origin($row["name"],$row["domain"]);
      }
      $records[] = $row;
    }
    mysql_free_result($result);
  }
  return $records;
}

/**
 * adds a record to an existing domain
 *
 * if valid domain-id and record information have been
 * passed to it, the function adds a new record to the
 * database and also creates a history entry
 * @param integer $zone the domain-id
 * @param string $name FQDN
 * @param string $type record-type (e.g. A)
 * @param string $content the content (e.g. IP Address)
 * @param integer $prio the MX priority (if unused, set to 0)
 * @param integer $ttl the TTL of this record (optional)
 * @returin integer primary key of mysql insert operation
 * @author Rudolph Bott
 * @copyright Copyright 2007, Rudolph Bott
 * @version V1.0
*/
function add_domain_record($zone,$name,$type,$content,$prio,$ttl = -1) {
  global $mysqlcfg;
  global $cfg;

  $tstamp = date("Ymd",time());
  
  if($ttl == -1) $ttl = $cfg["defaultttl"];

  if(!empty($zone) && !empty($type) && !empty($content) && !empty($ttl) && is_numeric($zone) && is_numeric($prio) && is_numeric($ttl)) {
    if(get_domain_type($zone) == 'SLAVE') {
      add_error(ERR_NO_EDIT_SLAVE,__FUNCTION__);
      return false;
    } 

    if($cfg["striporigin"]) {
      $domain = get_domain_from_id($zone);
      if(!empty($name)) {
        $name = $name.".".$domain;
      }
      else {
        $name = $domain;
      }
    }

    $query = "INSERT INTO ".$mysqlcfg["rectbl"]." (domain_id,name,type,content,prio,ttl,change_date) VALUES ('".mysql_real_escape_string($zone)."','".mysql_real_escape_string($name)."','".mysql_real_escape_string($type)."','".mysql_real_escape_string($content)."','".mysql_real_escape_string($prio)."','".mysql_real_escape_string($ttl)."','".$tstamp."')";
    mysql_query($query);
    
    $error = mysql_errno();
    $id = mysql_insert_id();

    if($error == 0) add_history($cfg["user"],'ADD RECORD',get_domain_from_id($zone));

    return $id;
  }
  else {
    return 0;
  }
}

/**
 * alters a record in the database
 *
 * the function alters a comment in the database
 * if valid zone and record information have been passed to it
 * @param integer $zone the domain-id
 * @param integer $record the record-id
 * @param string $name FQDN
 * @param string $type record-type (e.g. A)
 * @param string $content the content (e.g. IP Address)
 * @param integer $prio the MX priority (if unused, set to 0)
 * @param integer $ttl the TTL of this record (optional)
 * @return integer mysql status of last operation
 * @author Rudolph Bott
 * @copyright Copyright 2007, Rudolph Bott
 * @version V1.0
*/
function update_domain_record($zone,$record,$name,$type,$content,$prio,$ttl = 0) {
  global $mysqlcfg;
  global $cfg;

  $tstamp = date("Ymd",time());

  if($ttl == 0) $ttl = $cfg["defaultttl"];

  if(!empty($zone) && !empty($record) && !empty($type) && !empty($content) && !empty($ttl) && is_numeric($zone) && is_numeric($record) && is_numeric($prio) && is_numeric($ttl)) {

    if($cfg["striporigin"]) {
      $domain = get_domain_from_id($zone);
      if(!empty($name)) {
        $name = $name.".".$domain;
      }
      else {
        $name = $domain;
      }
    }

    $query = "UPDATE ".$mysqlcfg["rectbl"]." SET name='".mysql_real_escape_string($name)."', type='".mysql_real_escape_string($type)."', content='".mysql_real_escape_string($content)."', prio='".mysql_real_escape_string($prio)."', ttl='".mysql_real_escape_string($ttl)."', change_date='".$tstamp."' WHERE id='".mysql_real_escape_string($record)."' AND domain_id='".mysql_real_escape_string($zone)."'";
    mysql_query($query);
    
    $error = mysql_errno();

    if($error == 0) add_history($cfg["user"],'UPDATE RECORD',get_domain_from_id($zone));

    return $error;
  }
  else {
    return false;
  }
}

/**
 * deletes a record from the database
 *
 * deletes a record from the database via
 * via reference with record- and domain-id (as a security feature)
 * it also stores the record-structure in serialized form as
 * a history entry to make later restores possible
 * @param integer $rec_id the record-id
 * @param integer $zone_id the domain-id
 * @return integer returns status of last mysql operation (0 = OK)
 * @author Rudolph Bott
 * @copyright Copyright 2007, Rudolph Bott
 * @version V1.0
*/
function del_domain_record($rec_id,$zone_id) {
  global $mysqlcfg;
  global $cfg;

  if((!empty($rec_id) && !empty($zone_id) && (is_numeric($rec_id) && is_numeric($zone_id)))) {

    if(get_domain_type($zone_id) == 'SLAVE') {
      add_error(ERR_NO_EDIT_SLAVE,__FUNCTION__);
      return false;
    }

    $history[] = get_domain_record($rec_id);


    $query = "DELETE FROM ".$mysqlcfg["rectbl"]." WHERE id='".mysql_real_escape_string($rec_id)."' AND domain_id='".mysql_real_escape_string($zone_id)."' LIMIT 1";
    mysql_query($query);

    $error = mysql_errno();
    if($error == 0) add_history($cfg["user"],'DEL RECORD',get_domain_from_id($zone_id),$history);

    return $error;
  }
  else {
    return false;
  }
}

/**
 * checks if a given host has references in other records
 *
 * this function checks if a given host is being
 * referenced by other records (e.g. to ensure you
 * do not delete an A record which is still referenced
 * by MX, NS or CNAME records)
 * @param string $host the hostname to check
 * @return boolean true if referenced, otherwise false
 * @author Rudolph Bott
 * @copyright Copyright 2007, Rudolph Bott
 * @version 1.0
*/
function has_references($host) {
  global $mysqlcfg;

  $result = true;

  if(!empty($host)) {
    $query = "SELECT COUNT(*) FROM ".$mysqlcfg["rectbl"]." WHERE content = '".mysql_real_escape_string($host)."'";
    $result = mysql_query($query);

    $row = mysql_fetch_row($result);
    if($row[0] > 0) $result = true;
    else $result = false;
  }
  return $result;
}


/**
 * retrieves all records which are references
 * for the given host
 *
 * function to fetch all records which are a reference 
 * to the given host
 * this function is usefull when you want to remind poeple
 * that they are about to delete an A record which is still
 * referenced in CNAME's or MX's etc.
 * @param string $host
 * @return array multidimensional array records
 * @author Rudolph Bott
 * @copyright Copyright 2007, Rudolph Bott
 * @version V1.0
*/
function get_host_references($host) {
  global $mysqlcfg;

  $refs = array();

  if(!empty($host)) {
    $query = "SELECT * FROM ".$mysqlcfg["rectbl"]." WHERE content = '".mysql_real_escape_string($host)."'";
    
    $result = mysql_query($query);
    while($row = mysql_fetch_assoc($result)) {
      $refs[] = $row;
    }

    mysql_free_result($result);

  }

  return $refs;
}


/**
 * function to 're-sort' a record-array
 * containing reverse pointer records
 *
 * to get a proper display of reverse zones
 * use this function to apply a better
 * sorting to an array of records (as returned
 * by get_domain_records() )
 *
 * @param array $records array with record structures
 * @return array sorted array with record structs
 * @author Rudolph Bott
 * @copyright Copyright 2007, Rudolph Bott
 * @version V1.0
*/

function sort_reverse_zone_array($records) {
  global $cfg;

  $newlist = array(); 

  if($cfg["striporigin"]) {
    for($i = 0;$i < count($records);$i++) {
      $counter = 1;
      $key = "1".$records[$i]["name"].$counter;
      while(array_key_exists($key,$newlist)) {
        $counter++;
        $key = "1".$records[$i]["name"].$counter;
      }
      $newlist[$key] = $records[$i];
    }
  }
  else {
    for($i = 0;$i < count($records);$i++) {
      //basically, we chop of the .in-addr.arpa part
      //then we explode the reverse-ip-adress-part of the hostname into
      //its parts (e.g. 1, 0, 168 and 192)

      //next we re-arrange it as a number (all digits added up
      //to a length of three) in reverse ('normal') order: 192168000001

      //now we add the current record to a new array, using the
      //newly created number as its key
      $octets = substr($records[$i]["name"],0,-13);
      @list($fourth,$third,$second,$first) = explode(".",$octets);
      $key = addzeros($first,3) . addzeros($second,3) . addzeros($third,3) . addzeros($fourth,3);

      //if our generated key already exists, increase it as long as it takes ;)
      if(array_key_exists($key,$newlist)) {
        while(array_key_exists($key,$newlist)) $key++;
      }
      $newlist[$key] = $records[$i];
    }
  }
  //now use key-sort
  ksort($newlist);
  
  //done!
  return array_values($newlist);
}


//nueztliche record-funtkionen fuer templates


//einzelnen record per id aus DB holen
function get_template_record($id) {
  global $mysqlcfg;

  $record = array();

  if(!empty($id) && is_numeric($id)) {
    $query = "SELECT * FROM ".$mysqlcfg["tplrectbl"]." WHERE id='".mysql_real_escape_string($id)."'";
    $result = mysql_query($query);
    $record = mysql_fetch_assoc($result);
  }
  return $record;
}

//alle records eines templates aus der DB holen (in Reihenfolge SOA,NS,MX,TXT,A,CNAME)
function get_template_records($id) {
  global $mysqlcfg;

  $records = array();

  if(!empty($id) && is_numeric($id)) {
    $query = "SELECT id,name,type,prio,content,ttl,FIND_IN_SET(type,'SOA,NS,MX,TXT,PTR,A,CNAME') AS sortOrder FROM ".$mysqlcfg["tplrectbl"]." WHERE template_id='".mysql_real_escape_string($id)."' ORDER BY sortOrder,name";
    $result = mysql_query($query);
    while($row = mysql_fetch_assoc($result)) {
      $records[] = $row;
    }
    mysql_free_result($result);
  }
  return $records;
}

//alle records eines templates eines bestimmten typs aus der DB holen
function get_template_records_by_type($id,$type) {
  global $mysqlcfg;

  $records = array();
  return $records;
}

//einen record anhand der template-id hinzufuegen
function add_template_record($tpl,$name,$type,$content,$prio,$ttl = 0) {
  global $mysqlcfg;
  global $cfg;

  if($ttl == 0) $ttl = $cfg["defaultttl"];

  if(!empty($tpl) && !empty($name) && !empty($type) && !empty($content) && is_numeric($tpl) && is_numeric($prio) && is_numeric($ttl)) {
    $query = "INSERT INTO ".$mysqlcfg["tplrectbl"]." (template_id,name,type,content,prio,ttl) VALUES ('".mysql_real_escape_string($tpl)."','".mysql_real_escape_string($name)."','".mysql_real_escape_string($type)."','".mysql_real_escape_string($content)."','".mysql_real_escape_string($prio)."', '".mysql_real_escape_string($ttl)."')";
    mysql_query($query);
    return mysql_errno();
  }
  else {
    return false;
  }
}


/**
 * alters a template-record in the database
 *
 * @param integer $template the template-id
 * @param integer $record the record-id
 * @param string $name FQDN
 * @param string $type record-type (e.g. A)
 * @param string $content the content (e.g. IP Address)
 * @param integer $prio the MX priority (if unused, set to 0)
 * @param integer $ttl the TTL of this record (optional)
 * @return integer mysql status of last operation
 * @author Rudolph Bott
 * @copyright Copyright 2007, Rudolph Bott
 * @version V1.0
*/
function update_template_record($template,$record,$name,$type,$content,$prio,$ttl = 0) {
  global $mysqlcfg;
  global $cfg;

  $tstamp = date("Ymd",time());

  if($ttl == 0) $ttl = $cfg["defaultttl"];

  if(!empty($template) && !empty($record) && !empty($type) && !empty($content) && !empty($ttl) && is_numeric($template) && is_numeric($record) && is_numeric($prio) && is_numeric($ttl)) {

    $query = "UPDATE ".$mysqlcfg["tplrectbl"]." SET name='".mysql_real_escape_string($name)."', type='".mysql_real_escape_string($type)."', content='".mysql_real_escape_string($content)."', prio='".mysql_real_escape_string($prio)."', ttl='".mysql_real_escape_string($ttl)."' WHERE id='".mysql_real_escape_string($record)."' AND template_id='".mysql_real_escape_string($template)."'";
    mysql_query($query);

    $error = mysql_errno();

    return $error;
  }
  else {
    return false;
  }
}

//einen record anhand der template-id und record-id loeschen
function del_template_record($rec_id,$tpl_id) {
  global $mysqlcfg;

  if((!empty($rec_id) && !empty($tpl_id) && (is_numeric($rec_id) && is_numeric($tpl_id)))) {
    $query = "DELETE FROM ".$mysqlcfg["tplrectbl"]." WHERE id='".mysql_real_escape_string($rec_id)."' AND template_id='".mysql_real_escape_string($tpl_id)."' LIMIT 1";
    mysql_query($query);
    return mysql_errno();
  }
  else {
    return false;
  }
}

?>
